"use strict";
/**
 * Copyright (c) 2017, Daniel Imms (MIT License).
 * Copyright (c) 2018, Microsoft Corporation (MIT License).
 */
Object.defineProperty(exports, "__esModule", { value: true });
var unixTerminal_1 = require("./unixTerminal");
var assert = require("assert");
var cp = require("child_process");
var path = require("path");
var tty = require("tty");
var fs = require("fs");
var os_1 = require("os");
var testUtils_test_1 = require("./testUtils.test");
var FIXTURES_PATH = path.normalize(path.join(__dirname, '..', 'fixtures', 'utf8-character.txt'));
if (process.platform !== 'win32') {
    describe('UnixTerminal', function () {
        describe('Constructor', function () {
            it('should set a valid pts name', function () {
                var term = new unixTerminal_1.UnixTerminal('/bin/bash', [], {});
                var regExp;
                if (process.platform === 'linux') {
                    // https://linux.die.net/man/4/pts
                    regExp = /^\/dev\/pts\/\d+$/;
                }
                if (process.platform === 'darwin') {
                    // https://developer.apple.com/legacy/library/documentation/Darwin/Reference/ManPages/man4/pty.4.html
                    regExp = /^\/dev\/tty[p-sP-S][a-z0-9]+$/;
                }
                if (regExp) {
                    assert.ok(regExp.test(term.ptsName), '"' + term.ptsName + '" should match ' + regExp.toString());
                }
                assert.ok(tty.isatty(term.fd));
            });
        });
        describe('PtyForkEncodingOption', function () {
            it('should default to utf8', function (done) {
                var term = new unixTerminal_1.UnixTerminal('/bin/bash', ['-c', "cat \"".concat(FIXTURES_PATH, "\"")]);
                term.on('data', function (data) {
                    assert.strictEqual(typeof data, 'string');
                    assert.strictEqual(data, '\u00E6');
                    done();
                });
            });
            it('should return a Buffer when encoding is null', function (done) {
                var term = new unixTerminal_1.UnixTerminal('/bin/bash', ['-c', "cat \"".concat(FIXTURES_PATH, "\"")], {
                    encoding: null
                });
                term.on('data', function (data) {
                    assert.strictEqual(typeof data, 'object');
                    assert.ok(data instanceof Buffer);
                    assert.strictEqual(0xC3, data[0]);
                    assert.strictEqual(0xA6, data[1]);
                    done();
                });
            });
            it('should support other encodings', function (done) {
                var text = 'test æ!';
                var term = new unixTerminal_1.UnixTerminal(undefined, ['-c', 'echo "' + text + '"'], {
                    encoding: 'base64'
                });
                var buffer = '';
                term.onData(function (data) {
                    assert.strictEqual(typeof data, 'string');
                    buffer += data;
                });
                term.onExit(function () {
                    assert.strictEqual(Buffer.alloc(8, buffer, 'base64').toString().replace('\r', '').replace('\n', ''), text);
                    done();
                });
            });
        });
        describe('open', function () {
            var term;
            afterEach(function () {
                if (term) {
                    term.slave.destroy();
                    term.master.destroy();
                }
            });
            it('should open a pty with access to a master and slave socket', function (done) {
                term = unixTerminal_1.UnixTerminal.open({});
                var slavebuf = '';
                term.slave.on('data', function (data) {
                    slavebuf += data;
                });
                var masterbuf = '';
                term.master.on('data', function (data) {
                    masterbuf += data;
                });
                (0, testUtils_test_1.pollUntil)(function () {
                    if (masterbuf === 'slave\r\nmaster\r\n' && slavebuf === 'master\n') {
                        done();
                        return true;
                    }
                    return false;
                }, 200, 10);
                term.slave.write('slave\n');
                term.master.write('master\n');
            });
        });
        describe('close', function () {
            var term = new unixTerminal_1.UnixTerminal('node');
            it('should exit when terminal is destroyed programmatically', function (done) {
                term.on('exit', function (code, signal) {
                    assert.strictEqual(code, 0);
                    assert.strictEqual(signal, os_1.constants.signals.SIGHUP);
                    done();
                });
                term.destroy();
            });
        });
        /*
        describe('signals in parent and child', () => {
          it('SIGINT - custom in parent and child', done => {
            // this test is cumbersome - we have to run it in a sub process to
            // see behavior of SIGINT handlers
            const data = `
            var pty = require('./lib/index');
            process.on('SIGINT', () => console.log('SIGINT in parent'));
            var ptyProcess = pty.spawn('node', ['-e', 'process.on("SIGINT", ()=>console.log("SIGINT in child"));setTimeout(() => null, 300);'], {
              name: 'xterm-color',
              cols: 80,
              rows: 30,
              cwd: process.env.HOME,
              env: process.env
            });
            ptyProcess.on('data', function (data) {
              console.log(data);
            });
            setTimeout(() => null, 500);
            console.log('ready', ptyProcess.pid);
            `;
            const buffer: string[] = [];
            const p = cp.spawn('node', ['-e', data]);
            let sub = '';
            p.stdout.on('data', (data) => {
              if (!data.toString().indexOf('ready')) {
                sub = data.toString().split(' ')[1].slice(0, -1);
                setTimeout(() => {
                  process.kill(parseInt(sub), 'SIGINT');  // SIGINT to child
                  p.kill('SIGINT');                       // SIGINT to parent
                }, 200);
              } else {
                buffer.push(data.toString().replace(/^\s+|\s+$/g, ''));
              }
            });
            p.on('close', () => {
              // handlers in parent and child should have been triggered
              assert.strictEqual(buffer.indexOf('SIGINT in child') !== -1, true);
              assert.strictEqual(buffer.indexOf('SIGINT in parent') !== -1, true);
              done();
            });
          });
          it('SIGINT - custom in parent, default in child', done => {
            // this tests the original idea of the signal(...) change in pty.cc:
            // to make sure the SIGINT handler of a pty child is reset to default
            // and does not interfere with the handler in the parent
            const data = `
            var pty = require('./lib/index');
            process.on('SIGINT', () => console.log('SIGINT in parent'));
            var ptyProcess = pty.spawn('node', ['-e', 'setTimeout(() => console.log("should not be printed"), 300);'], {
              name: 'xterm-color',
              cols: 80,
              rows: 30,
              cwd: process.env.HOME,
              env: process.env
            });
            ptyProcess.on('data', function (data) {
              console.log(data);
            });
            setTimeout(() => null, 500);
            console.log('ready', ptyProcess.pid);
            `;
            const buffer: string[] = [];
            const p = cp.spawn('node', ['-e', data]);
            let sub = '';
            p.stdout.on('data', (data) => {
              if (!data.toString().indexOf('ready')) {
                sub = data.toString().split(' ')[1].slice(0, -1);
                setTimeout(() => {
                  process.kill(parseInt(sub), 'SIGINT');  // SIGINT to child
                  p.kill('SIGINT');                       // SIGINT to parent
                }, 200);
              } else {
                buffer.push(data.toString().replace(/^\s+|\s+$/g, ''));
              }
            });
            p.on('close', () => {
              // handlers in parent and child should have been triggered
              assert.strictEqual(buffer.indexOf('should not be printed') !== -1, false);
              assert.strictEqual(buffer.indexOf('SIGINT in parent') !== -1, true);
              done();
            });
          });
          it('SIGHUP default (child only)', done => {
            const term = new UnixTerminal('node', [ '-e', `
            console.log('ready');
            setTimeout(()=>console.log('timeout'), 200);`
            ]);
            let buffer = '';
            term.on('data', (data) => {
              if (data === 'ready\r\n') {
                term.kill();
              } else {
                buffer += data;
              }
            });
            term.on('exit', () => {
              // no timeout in buffer
              assert.strictEqual(buffer, '');
              done();
            });
          });
          it('SIGUSR1 - custom in parent and child', done => {
            let pHandlerCalled = 0;
            const handleSigUsr = function(h: any): any {
              return function(): void {
                pHandlerCalled += 1;
                process.removeListener('SIGUSR1', h);
              };
            };
            process.on('SIGUSR1', handleSigUsr(handleSigUsr));
    
            const term = new UnixTerminal('node', [ '-e', `
            process.on('SIGUSR1', () => {
              console.log('SIGUSR1 in child');
            });
            console.log('ready');
            setTimeout(()=>null, 200);`
            ]);
            let buffer = '';
            term.on('data', (data) => {
              if (data === 'ready\r\n') {
                process.kill(process.pid, 'SIGUSR1');
                term.kill('SIGUSR1');
              } else {
                buffer += data;
              }
            });
            term.on('exit', () => {
              // should have called both handlers and only once
              assert.strictEqual(pHandlerCalled, 1);
              assert.strictEqual(buffer, 'SIGUSR1 in child\r\n');
              done();
            });
          });
        });
        */
        describe('spawn', function () {
            if (process.platform === 'darwin') {
                it('should return the name of the process', function (done) {
                    var term = new unixTerminal_1.UnixTerminal('/bin/echo');
                    assert.strictEqual(term.process, '/bin/echo');
                    term.on('exit', function () { return done(); });
                    term.destroy();
                });
                it('should close on exec', function (done) {
                    var data = "\n          var pty = require('./lib/index');\n          var ptyProcess = pty.spawn('node', ['-e', 'setTimeout(() => console.log(\"hello from terminal\"), 300);']);\n          ptyProcess.on('data', function (data) {\n            console.log(data);\n          });\n          setTimeout(() => null, 500);\n          console.log('ready', ptyProcess.pid);\n          ";
                    var buffer = [];
                    var readFd = fs.openSync(FIXTURES_PATH, 'r');
                    var p = cp.spawn('node', ['-e', data], {
                        stdio: ['ignore', 'pipe', 'pipe', readFd]
                    });
                    var sub = '';
                    p.stdout.on('data', function (data) {
                        if (!data.toString().indexOf('ready')) {
                            sub = data.toString().split(' ')[1].slice(0, -1);
                            try {
                                fs.statSync("/proc/".concat(sub, "/fd/").concat(readFd));
                                done('not reachable');
                            }
                            catch (error) {
                                // @ts-ignore
                                assert.notStrictEqual(error.message.indexOf('ENOENT'), -1);
                            }
                            setTimeout(function () {
                                process.kill(parseInt(sub), 'SIGINT'); // SIGINT to child
                                p.kill('SIGINT'); // SIGINT to parent
                            }, 200);
                        }
                        else {
                            buffer.push(data.toString().replace(/^\s+|\s+$/g, ''));
                        }
                    });
                    p.on('close', function () {
                        done();
                    });
                });
            }
            it('should handle exec() errors', function (done) {
                var term = new unixTerminal_1.UnixTerminal('/bin/bogus.exe', []);
                term.on('exit', function (code, signal) {
                    assert.strictEqual(code, 1);
                    done();
                });
            });
            it('should handle chdir() errors', function (done) {
                var term = new unixTerminal_1.UnixTerminal('/bin/echo', [], { cwd: '/nowhere' });
                term.on('exit', function (code, signal) {
                    assert.strictEqual(code, 1);
                    done();
                });
            });
            /*
            it('should not leak child process', (done) => {
              const count = cp.execSync('ps -ax | grep node | wc -l');
              const term = new UnixTerminal('node', [ '-e', `
                console.log('ready');
                setTimeout(()=>console.log('timeout'), 200);`
              ]);
              term.on('data', async (data) => {
                if (data === 'ready\r\n') {
                  process.kill(term.pid, 'SIGINT');
                  await setTimeout(() => null, 1000);
                  const newCount = cp.execSync('ps -ax | grep node | wc -l');
                  assert.strictEqual(count.toString(), newCount.toString());
                  done();
                }
              });
            });
            */
        });
    });
}
//# sourceMappingURL=unixTerminal.test.js.map